/******************************************************************************

   main.c
   Copyright (c) 1996-1998, Soundcode, Inc. All rights reserved.

   This code and information is provided "as is" without warranty 
   of any kind, either expressed or implied, including but not 
   limited to the implied warranties of merchantability and/or 
   fitness for a particular purpose. 

******************************************************************************/

/* System headers */
#include <windows.h>
#include <assert.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>

/* Public headers */
#include "scnsm.h"

/* Private headers */
#include "resource.h"

/* Pragmas */
#pragma warning(disable : 4704)  /* in-line assembler precludes global optimizations */

/* Constants */
#define APPNAME "SCNSM Protected Mode Test"
#define DRIVER  "SCNSM   "

/* Functions */
DWORD Open();
WORD GetVer(DWORD hNSM, DWORD* pdwVer);
WORD GetPageSize(DWORD hNSM, WORD* pwPageSize);
WORD AllocPage(DWORD hNSM, WORD nPages, char far** ppBuf, DWORD* phBuf);
WORD FreePage(DWORD hNSM, DWORD hBuf);

long FAR PASCAL WndProc(HWND, UINT, UINT, LONG);
void Printf(char *fmt, ...);

/* Globals */
DWORD g_hNSM = 0;
DWORD g_hBuf = 0;
char far* g_pBuf = 0;
HANDLE g_hInstance;
HWND g_hWnd;

/* Test */
static void DoTest()
{
   WORD rc = 0;
   char szMsg[512] = {0};
   WORD cbPage;
   DWORD dwVer;

   /* Get API address */
   g_hNSM = Open();
   if( !g_hNSM ) 
   {
      Printf("Cannot load %s\n", DRIVER);
      goto exit;
   }

   /* Version */
   rc = GetVer(g_hNSM, &dwVer);
   if( rc )
   {
      Printf("NSM_GETVERSION failed (%d)\n", rc);
      goto exit;
   }
   sprintf(szMsg, "%s version %u.%u\n", DRIVER, HIWORD(dwVer), LOWORD(dwVer));

   /* Page size */
   rc = GetPageSize(g_hNSM, &cbPage);
   if( rc )
   {
      Printf("NSM_GETPAGESIZE failed (%d)\n", rc);
      goto exit;
   }
   sprintf(szMsg, "%s\tpage size = %d\n", szMsg, cbPage);

   /* Allocate page */
   {
      WORD nPages = 1;
      rc = AllocPage(g_hNSM, nPages, &g_pBuf, &g_hBuf);
      if( rc )
      {
         Printf("NSM_ALLOCPAGE failed (%d)\n", rc);
         goto exit;
      }
      sprintf(szMsg, "%s\tpBuf = %lx\n", szMsg, g_pBuf);
      sprintf(szMsg, "%s\thBuf = %lx\n", szMsg, g_hBuf);
      memset(g_pBuf, '=', nPages*cbPage);
      g_pBuf[20] = 0;
      sprintf(szMsg, "%s\t%s\n", szMsg, g_pBuf);
      strcpy(g_pBuf, "Just testing!");
      sprintf(szMsg, "%s\t%s\n", szMsg, g_pBuf);
      Printf("%s", szMsg); *szMsg = 0;
   }

exit:
   ;
}

/* Open driver */
static DWORD Open()
{
   DWORD hNSM = 0;

#ifdef WIN31COMPAT
   _asm
   {
      mov bx, SCNSM_DEVICE_ID
      mov ax, 1684h
      int 2fh
      mov word ptr hNSM, di
      mov word ptr hNSM+2, es
   }
#else
   char szDriver[9];
   strncpy(szDriver, DRIVER, sizeof(szDriver));
   _asm
   {
      xor bx, bx
      mov di, ss
      mov es, di
      lea di, szDriver
      mov ax, 1684h
      int 2fh
      mov word ptr hNSM, di
      mov word ptr hNSM+2, es
   }
#endif
   return hNSM;
}

/* Get version */
static WORD GetVer(DWORD hNSM, DWORD* pdwVer)
{
   WORD rc;
   DWORD dwVer;
   WORD cbVer = sizeof(dwVer);

   assert(hNSM);
   assert(pdwVer);

   _asm
   {
      mov ax, NSM_GETVERSION
      lea bx, dwVer
      mov cx, cbVer
      call [hNSM]
      mov rc, ax
   }
   *pdwVer = dwVer;
   return rc;
}

/* GetPageSize */
static WORD GetPageSize(DWORD hNSM, WORD* pwcbPage)
{
   WORD rc;
   WORD cbPage;
   WORD cbcbPage = sizeof(cbPage);

   assert(hNSM);
   assert(pwcbPage);

   _asm
   {
      mov ax, NSM_GETPAGESIZE
      lea bx, cbPage
      mov cx, cbcbPage
      call [hNSM]
      mov rc, ax
   }
   *pwcbPage = cbPage;
   return rc;
}

/* Allocate/Free */
static WORD AllocPage(DWORD hNSM, WORD nPages, char far** ppBuf, DWORD* phBuf)
{
   WORD rc;
   DWORD _hBuf;
   char far* _pBuf;

   assert(hNSM);
   assert(phBuf);
   assert(ppBuf);
   assert(nPages >= 1);

   _asm
   {
      mov ax, NSM_ALLOCPAGE
      mov bx, nPages
      lea cx, _pBuf
      lea dx, _hBuf
      call [hNSM]
      mov rc, ax
   }
   *phBuf = _hBuf;
   *ppBuf = _pBuf;
   return rc;
}

static WORD FreePage(DWORD hNSM, DWORD hBuf)
{
   WORD rc;

   assert(hNSM);
   assert(hBuf);

   _asm
   {
      mov ax, NSM_FREEPAGE
      lea bx, hBuf
      call [hNSM]
      mov rc, ax
   }
   return rc;
}

/* Printf */
static void Printf(char *fmt, ...)
{
   va_list args;
   char szMessage[512];

   va_start(args, fmt);
   vsprintf(szMessage, fmt, args);
   va_end(args);
   MessageBox(g_hWnd, szMessage, APPNAME, MB_OK);
}

/* Windows function implementations */
int PASCAL WinMain(HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpszCmdParam, int nCmdShow)
{
   static char szAppName[] = APPNAME;
   MSG msg;
   WNDCLASS wndclass;

   if( !hPrevInstance )
   {
      wndclass.style         = CS_HREDRAW | CS_VREDRAW;
      wndclass.lpfnWndProc   = WndProc;
      wndclass.cbClsExtra    = 0;
      wndclass.cbWndExtra    = 0;
      wndclass.hInstance     = hInstance;
      wndclass.hIcon         = LoadIcon(NULL, MAKEINTRESOURCE(IDI_TEST));
      wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
      wndclass.hbrBackground = GetStockObject(LTGRAY_BRUSH);
      wndclass.lpszMenuName  = NULL;
      wndclass.lpszClassName = szAppName;

      RegisterClass(&wndclass);
   }

   g_hWnd = CreateWindow(szAppName,
      APPNAME,
      WS_OVERLAPPEDWINDOW,
      50,
      50,
      640,
      480,
      NULL,
      LoadMenu(hInstance, MAKEINTRESOURCE(IDM_MENU)),
      hInstance,
      NULL);

   ShowWindow(g_hWnd, nCmdShow);
   UpdateWindow(g_hWnd);
     
   while( GetMessage(&msg, NULL, 0, 0) )
   {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
   }
    
   return msg.wParam;
}

/* Windows procedure function */
long FAR PASCAL WndProc(HWND hWnd, UINT message, UINT wParam, LONG lParam)
{
   switch( message )
   {              
      case WM_CREATE:
      {
         g_hInstance = ((LPCREATESTRUCT)lParam)->hInstance;
         DoTest();
         //SendMessage(hWnd, WM_CLOSE, 0, 0);
         return 0;
      }
    
      case WM_COMMAND:
      {           
         switch(wParam)
         {
            case ID_FILE_EXIT:
               SendMessage(hWnd, WM_CLOSE, 0, 0);
               return 0;

            default:
               break;
         }
         return 0;
      }       

      case WM_CLOSE:
      {
         /* Free page */
         if( g_hNSM && g_hBuf )
         {
            WORD rc = FreePage(g_hNSM, g_hBuf);
            if( rc )
               Printf("NSM_FREEPAGE failed (%d)\n", rc);
         }
      }

      case WM_DESTROY:
      {
         PostQuitMessage(0); 
         return 0;
      }
   }   
   return DefWindowProc(hWnd, message, wParam, lParam);
}

