/******************************************************************************

   main.c
   Copyright (c) 1996-1998, Soundcode, Inc. All rights reserved.

   This code and information is provided "as is" without warranty 
   of any kind, either expressed or implied, including but not 
   limited to the implied warranties of merchantability and/or 
   fitness for a particular purpose. 

******************************************************************************/

/* System headers */
#include <assert.h>
#include <stdio.h>
#include <memory.h>
#include <string.h>

/* Public headers */
#include "scnsm.h"

/* Pragmas */
#pragma warning(disable : 4704)  /* in-line assembler precludes global optimizations */

/* Constants */
#define DRIVER   "SCNSM   "

/* Typedefs */
typedef unsigned short WORD;
typedef unsigned long DWORD;

/* Macros */
#define LOWORD(l)    ((WORD)(l))
#define HIWORD(l)    ((WORD)(((DWORD)(l) >> 16) & 0xFFFF))

/* Functions */
DWORD Open();
WORD GetVer(DWORD hNSM, DWORD* pdwVer);
WORD GetPageSize(DWORD hNSM, WORD* pwPageSize);
WORD AllocPage(DWORD hNSM, WORD nPages, char far** ppBuf, DWORD* phBuf);
WORD FreePage(DWORD hNSM, DWORD hBuf);

/* Test */
static void DoTest()
{
   DWORD hNSM = 0;
   DWORD hBuf = 0;
   char far* pBuf = 0;

   WORD rc = 0;
   WORD cbPage;
   DWORD dwVer;

   /* Get API address */
   hNSM = Open();
   if( !hNSM ) 
   {
      printf("Cannot load %s\n", DRIVER);
      goto exit;
   }

   /* Version */
   rc = GetVer(hNSM, &dwVer);
   if( rc )
   {
      printf("NSM_GETVERSION failed (%d)\n", rc);
      goto exit;
   }
   printf("%s version %u.%u\n", DRIVER, HIWORD(dwVer), LOWORD(dwVer));

   /* Page size */
   rc = GetPageSize(hNSM, &cbPage);
   if( rc )
   {
      printf("NSM_GETPAGESIZE failed (%d)\n", rc);
      goto exit;
   }
   printf("\tpage size = %d\n", cbPage);

   /* Allocate page */
   {
      WORD nPages = 1;
      rc = AllocPage(hNSM, nPages, &pBuf, &hBuf);
      if( rc )
      {
         printf("NSM_ALLOCPAGE failed (%d)\n", rc);
         goto exit;
      }
      printf("\tpBuf = %lx\n", pBuf);
      printf("\thBuf = %lx\n", hBuf);
      memset(pBuf, '=', nPages*cbPage);
      pBuf[20] = 0;
      printf("\t%s\n", pBuf);
      strcpy(pBuf, "Just testing!");
      printf("\t%s\n", pBuf);
   }

   /* Free page */
   rc = FreePage(hNSM, hBuf);
   if( rc )
   {
      printf("NSM_FREEPAGE failed (%d)\n", rc);
      goto exit;
   }

exit:
   ;
}

/* Open driver */
static DWORD Open()
{
   DWORD hNSM = 0;

#ifdef WIN31COMPAT
   _asm
   {
      mov bx, SCNSM_DEVICE_ID
      mov ax, 1684h
      int 2fh
      mov word ptr hNSM, di
      mov word ptr hNSM+2, es
   }
#else
   char szDriver[9];
   strncpy(szDriver, DRIVER, sizeof(szDriver));
   _asm
   {
      xor bx, bx
      mov di, ss
      mov es, di
      lea di, szDriver
      mov ax, 1684h
      int 2fh
      mov word ptr hNSM, di
      mov word ptr hNSM+2, es
   }
#endif
   return hNSM;
}

/* Get version */
static WORD GetVer(DWORD hNSM, DWORD* pdwVer)
{
   WORD rc;
   DWORD dwVer;
   WORD cbVer = sizeof(dwVer);

   assert(hNSM);
   assert(pdwVer);

   _asm
   {
      mov ax, NSM_GETVERSION
      lea bx, dwVer
      mov cx, cbVer
      call [hNSM]
      mov rc, ax
   }
   *pdwVer = dwVer;
   return rc;
}

/* GetPageSize */
static WORD GetPageSize(DWORD hNSM, WORD* pwcbPage)
{
   WORD rc;
   WORD cbPage;
   WORD cbcbPage = sizeof(cbPage);

   assert(hNSM);
   assert(pwcbPage);

   _asm
   {
      mov ax, NSM_GETPAGESIZE
      lea bx, cbPage
      mov cx, cbcbPage
      call [hNSM]
      mov rc, ax
   }
   *pwcbPage = cbPage;
   return rc;
}

/* Allocate/Free */
static WORD AllocPage(DWORD hNSM, WORD nPages, char far** ppBuf, DWORD* phBuf)
{
   WORD rc;
   DWORD _hBuf;
   char far* _pBuf;

   assert(hNSM);
   assert(phBuf);
   assert(ppBuf);
   assert(nPages >= 1);

   _asm
   {
      mov ax, NSM_ALLOCPAGE
      mov bx, nPages
      lea cx, _pBuf
      lea dx, _hBuf
      call [hNSM]
      mov rc, ax
   }
   *phBuf = _hBuf;
   *ppBuf = _pBuf;
   return rc;
}

static WORD FreePage(DWORD hNSM, DWORD hBuf)
{
   WORD rc;

   assert(hNSM);
   assert(hBuf);

   _asm
   {
      mov ax, NSM_FREEPAGE
      lea bx, hBuf
      call [hNSM]
      mov rc, ax
   }
   return rc;
}

/* Main */
int main(int argc, char** argv)
{
   DoTest();
   return 0;
}
